package eu.headcrashing;

import java.lang.annotation.Annotation;
import java.lang.reflect.Type;
import java.time.Instant;
import java.time.format.DateTimeParseException;
import java.util.Optional;

import javax.ws.rs.ext.ParamConverter;
import javax.ws.rs.ext.ParamConverterProvider;
import javax.ws.rs.ext.Provider;

@Provider
public class InstantConversion implements ParamConverterProvider {
	@SuppressWarnings("unchecked")
	@Override
	public <T> ParamConverter<T> getConverter(final Class<T> rawType, final Type genericType, final Annotation[] annotations) {
		if (rawType != Instant.class)
			return null;

		return (ParamConverter<T>) new ParamConverter<Instant>() {
			@Override
			public Instant fromString(final String value) {
				try {
					return Optional.ofNullable(value).map(Instant::parse).orElse(null);
				} catch (final DateTimeParseException e) {
					throw new IllegalArgumentException(e);
				}
			}

			@Override
			public String toString(final Instant value) {
				return Optional.of(value).map(Instant::toString).orElse(null);
			}
		};
	}
}